@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -30,10 +30,10 @@
 #include <stdlib.h>
 #include <wget.h>
 
-int main(int argc G_GNUC_WGET_UNUSED, const char *const *argv G_GNUC_WGET_UNUSED)
+int main(int argc WGET_GCC_UNUSED, const char *const *argv WGET_GCC_UNUSED)
 {
-	wget_http_connection_t *conn = NULL;
-	wget_http_response_t *resp;
+	wget_http_connection *conn = NULL;
+	wget_http_response *resp;
 
 	// set up libwget global configuration
 	wget_global_init(
