@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -81,7 +81,7 @@ static void html_parse_localfile(const char *fname)
 			}
 		}
 
-		wget_html_parsed_result_t *res  = wget_html_get_urls_inline(data, NULL, NULL);
+		wget_html_parsed_result *res  = wget_html_get_urls_inline(data, NULL, NULL);
 
 		if (encoding) {
 			if (res->encoding && wget_strcasecmp_ascii(encoding, res->encoding))
@@ -89,8 +89,8 @@ static void html_parse_localfile(const char *fname)
 		}
 
 		for (int it = 0; it < wget_vector_size(res->uris); it++) {
-			wget_html_parsed_url_t *html_url = wget_vector_get(res->uris, it);
-			wget_string_t *url = &html_url->url;
+			wget_html_parsed_url *html_url = wget_vector_get(res->uris, it);
+			wget_string *url = &html_url->url;
 
 			printf("  %s.%s '%.*s'\n", html_url->dir, html_url->attr, (int) url->len, url->p);
 		}
