@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2017-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -29,7 +29,7 @@
 
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 {
-	wget_bar_t *bar;
+	wget_bar *bar;
 	char *in = (char *) malloc(size + 1);
 
 	assert(in != NULL);
@@ -46,7 +46,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 	bar = wget_bar_init(NULL, 3);
 	wget_bar_set_slots(bar, 2);
 
-	wget_bar_slot_begin(bar, 1, "test", 64000);
+	wget_bar_slot_begin(bar, 1, "test", 1, 64000);
 	wget_bar_slot_downloaded(bar, 1, (unsigned) atoi(in));
 	wget_bar_printf(bar, 1, "%s", in);
 	wget_bar_write_line(bar, (char *) data, size);
