@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2017-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -55,10 +55,10 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 	g_size = size;
 
 #if ! defined _WIN32 && defined HAVE_FMEMOPEN
-	wget_hpkp_db_t *hpkp_db = wget_hpkp_db_init(NULL, "hpkp");
+	wget_hpkp_db *hpkp_db = wget_hpkp_db_init(NULL, "hpkp");
 	wget_hpkp_db_load(hpkp_db);
 #else
-	wget_hpkp_db_t *hpkp_db = wget_hpkp_db_init(NULL, NULL);
+	wget_hpkp_db *hpkp_db = wget_hpkp_db_init(NULL, NULL);
 #endif
 	wget_hpkp_db_check_pubkey(hpkp_db, "x.y", "0", 1);
 	wget_hpkp_db_free(&hpkp_db);
