@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -54,13 +54,17 @@
 #include <brotli/decode.h>
 #endif
 
+#ifdef WITH_ZSTD
+#include <zstd.h>
+#endif
+
 #include <wget.h>
 #include "private.h"
 
-typedef int (*wget_decompressor_decompress_t)(wget_decompressor_t *dc, char *src, size_t srclen);
-typedef void (*wget_decompressor_exit_t)(wget_decompressor_t *dc);
+typedef int wget_decompressor_decompress_fn(wget_decompressor *dc, char *src, size_t srclen);
+typedef void wget_decompressor_exit_fn(wget_decompressor *dc);
 
-struct _wget_decompressor_st {
+struct wget_decompressor_st {
 #ifdef WITH_ZLIB
 	z_stream
 		z_strm;
@@ -77,18 +81,22 @@ struct _wget_decompressor_st {
 	BrotliDecoderState
 		*brotli_strm;
 #endif
+#ifdef WITH_ZSTD
+	ZSTD_DStream
+		*zstd_strm;
+#endif
 
-	wget_decompressor_sink_t
-		sink; // decompressed data goes here
-	wget_decompressor_error_handler_t
-		error_handler; // called on error
-	wget_decompressor_decompress_t
-		decompress;
-	wget_decompressor_exit_t
-		exit;
+	wget_decompressor_sink_fn
+		*sink; // decompressed data goes here
+	wget_decompressor_error_handler
+		*error_handler; // called on error
+	wget_decompressor_decompress_fn
+		*decompress;
+	wget_decompressor_exit_fn
+		*exit;
 	void
 		*context; // given to sink()
-	wget_content_encoding_type_t
+	wget_content_encoding
 		encoding;
 };
 
@@ -107,7 +115,7 @@ static int gzip_init(z_stream *strm)
 	return 0;
 }
 
-static int gzip_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
+static int gzip_decompress(wget_decompressor *dc, char *src, size_t srclen)
 {
 	z_stream *strm;
 	char dst[10240];
@@ -143,7 +151,7 @@ static int gzip_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
 	return -1;
 }
 
-static void gzip_exit(wget_decompressor_t *dc)
+static void gzip_exit(wget_decompressor *dc)
 {
 	int status;
 
@@ -179,7 +187,7 @@ static int lzma_init(lzma_stream *strm)
 	return 0;
 }
 
-static int lzma_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
+static int lzma_decompress(wget_decompressor *dc, char *src, size_t srclen)
 {
 	lzma_stream *strm;
 	char dst[10240];
@@ -215,7 +223,7 @@ static int lzma_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
 	return -1;
 }
 
-static void lzma_exit(wget_decompressor_t *dc)
+static void lzma_exit(wget_decompressor *dc)
 {
 	lzma_end(&dc->lzma_strm);
 }
@@ -232,7 +240,7 @@ static int brotli_init(BrotliDecoderState **strm)
 	return 0;
 }
 
-static int brotli_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
+static int brotli_decompress(wget_decompressor *dc, char *src, size_t srclen)
 {
 	BrotliDecoderState *strm;
 	BrotliDecoderResult status;
@@ -273,12 +281,70 @@ static int brotli_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
 	return -1;
 }
 
-static void brotli_exit(wget_decompressor_t *dc)
+static void brotli_exit(wget_decompressor *dc)
 {
 	BrotliDecoderDestroyInstance(dc->brotli_strm);
 }
 #endif // WITH_BROTLIDEC
 
+#ifdef WITH_ZSTD
+static int zstd_init(ZSTD_DStream **strm)
+{
+	if ((*strm = ZSTD_createDStream()) == NULL) {
+		error_printf(_("Failed to create Zstandard decompression\n"));
+		return -1;
+	}
+
+	size_t rc = ZSTD_initDStream(*strm);
+	if (ZSTD_isError(rc)) {
+		error_printf(_("Failed to init Zstandard decompression: %s\n"), ZSTD_getErrorName(rc));
+		ZSTD_freeDStream(*strm);
+		*strm = NULL;
+		return -1;
+	}
+
+	return 0;
+}
+
+static int zstd_decompress(wget_decompressor *dc, char *src, size_t srclen)
+{
+	ZSTD_DStream *strm;
+	uint8_t dst[10240];
+
+	if (!srclen) {
+		// special case to avoid decompress errors
+		if (dc->sink)
+			dc->sink(dc->context, "", 0);
+
+		return 0;
+	}
+
+	strm = dc->zstd_strm;
+
+	ZSTD_inBuffer input = { .src = src, .size = srclen, .pos = 0 };
+
+	while (input.pos < input.size) {
+		ZSTD_outBuffer output = { .dst = dst, .size = sizeof(dst), .pos = 0 };
+
+		size_t rc = ZSTD_decompressStream(strm, &output , &input);
+		if (ZSTD_isError(rc)) {
+			error_printf(_("Failed to init Zstandard decompression: %s\n"), ZSTD_getErrorName(rc));
+			return -1;
+		}
+
+		if (dc->sink)
+			dc->sink(dc->context, (char *)dst, output.pos);
+	}
+
+	return 0;
+}
+
+static void zstd_exit(wget_decompressor *dc)
+{
+	ZSTD_freeDStream(dc->zstd_strm);
+}
+#endif // WITH_ZSTD
+
 #ifdef WITH_BZIP2
 static int bzip2_init(bz_stream *strm)
 {
@@ -292,7 +358,7 @@ static int bzip2_init(bz_stream *strm)
 	return 0;
 }
 
-static int bzip2_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
+static int bzip2_decompress(wget_decompressor *dc, char *src, size_t srclen)
 {
 	bz_stream *strm;
 	char dst[10240];
@@ -328,13 +394,13 @@ static int bzip2_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
 	return -1;
 }
 
-static void bzip2_exit(wget_decompressor_t *dc)
+static void bzip2_exit(wget_decompressor *dc)
 {
 	BZ2_bzDecompressEnd(&dc->bz_strm);
 }
 #endif // WITH_BZIP2
 
-static int identity(wget_decompressor_t *dc, char *src, size_t srclen)
+static int identity(wget_decompressor *dc, char *src, size_t srclen)
 {
 	if (dc->sink)
 		dc->sink(dc->context, src, srclen);
@@ -342,13 +408,16 @@ static int identity(wget_decompressor_t *dc, char *src, size_t srclen)
 	return 0;
 }
 
-wget_decompressor_t *wget_decompress_open(
-	wget_content_encoding_type_t encoding,
-	wget_decompressor_sink_t sink,
+wget_decompressor *wget_decompress_open(
+	wget_content_encoding encoding,
+	wget_decompressor_sink_fn *sink,
 	void *context)
 {
-	wget_decompressor_t *dc = xcalloc(1, sizeof(wget_decompressor_t));
 	int rc = 0;
+	wget_decompressor *dc = wget_calloc(1, sizeof(wget_decompressor));
+
+	if (!dc)
+		return NULL;
 
 	if (encoding == wget_content_encoding_gzip) {
 #ifdef WITH_ZLIB
@@ -384,6 +453,13 @@ wget_decompressor_t *wget_decompress_open(
 			dc->decompress = brotli_decompress;
 			dc->exit = brotli_exit;
 		}
+#endif
+	} else if (encoding == wget_content_encoding_zstd) {
+#ifdef WITH_ZSTD
+		if ((rc = zstd_init(&dc->zstd_strm)) == 0) {
+			dc->decompress = zstd_decompress;
+			dc->exit = zstd_exit;
+		}
 #endif
 	}
 
@@ -404,7 +480,7 @@ wget_decompressor_t *wget_decompress_open(
 	return dc;
 }
 
-void wget_decompress_close(wget_decompressor_t *dc)
+void wget_decompress_close(wget_decompressor *dc)
 {
 	if (dc) {
 		if (dc->exit)
@@ -413,25 +489,25 @@ void wget_decompress_close(wget_decompressor_t *dc)
 	}
 }
 
-int wget_decompress(wget_decompressor_t *dc, char *src, size_t srclen)
+int wget_decompress(wget_decompressor *dc, char *src, size_t srclen)
 {
 	if (dc) {
 		int rc = dc->decompress(dc, src, srclen);
 
 		if (rc && dc->error_handler)
-			dc->error_handler(dc->context, rc);
+			dc->error_handler(dc, rc);
 	}
 
 	return 0;
 }
 
-void wget_decompress_set_error_handler(wget_decompressor_t *dc, wget_decompressor_error_handler_t error_handler)
+void wget_decompress_set_error_handler(wget_decompressor *dc, wget_decompressor_error_handler *error_handler)
 {
 	if (dc)
 		dc->error_handler = error_handler;
 }
 
-void *wget_decompress_get_context(wget_decompressor_t *dc)
+void *wget_decompress_get_context(wget_decompressor *dc)
 {
 	return dc ? dc->context : NULL;
 }
@@ -444,12 +520,13 @@ static char _encoding_names[wget_content_encoding_max][9] = {
 	[wget_content_encoding_lzma] = "lzma",
 	[wget_content_encoding_bzip2] = "bzip2",
 	[wget_content_encoding_brotli] = "br",
+	[wget_content_encoding_zstd] = "zstd",
 };
 
-wget_content_encoding_type_t wget_content_encoding_by_name(const char *name)
+wget_content_encoding wget_content_encoding_by_name(const char *name)
 {
 	if (name) {
-		for (wget_content_encoding_type_t it = 0; it < wget_content_encoding_max; it++) {
+		for (wget_content_encoding it = 0; it < wget_content_encoding_max; it++) {
 			if (!strcmp(_encoding_names[it], name))
 				return it;
 		}
@@ -461,7 +538,7 @@ wget_content_encoding_type_t wget_content_encoding_by_name(const char *name)
 	return wget_content_encoding_unknown;
 }
 
-const char *wget_content_encoding_to_name(wget_content_encoding_type_t type)
+const char *wget_content_encoding_to_name(wget_content_encoding type)
 {
 	if (type >= 0 && type < wget_content_encoding_max)
 		return _encoding_names[type];
