@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012-2015 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -38,12 +38,6 @@
 
 #if defined HAVE_IDN2_H && defined WITH_LIBIDN2
 # include <idn2.h>
-# if IDN2_VERSION_NUMBER < 0x00140000
-#  if defined HAVE_UNICASE_H && defined WITH_LIBUNISTRING
-#   include <unicase.h>
-#   include <unistr.h>
-#  endif
-# endif
 #elif defined HAVE_IDNA_H && defined WITH_LIBIDN
 # include <idna.h>
 # ifdef _WIN32
@@ -71,7 +65,7 @@ const char *wget_local_charset_encoding(void)
 int wget_memiconv(const char *src_encoding, const void *src, size_t srclen, const char *dst_encoding, char **out, size_t *outlen)
 {
 	if (!src)
-		return -1;
+		return WGET_E_INVALID;
 
 #ifdef HAVE_ICONV
 	if (!src_encoding)
@@ -80,14 +74,19 @@ int wget_memiconv(const char *src_encoding, const void *src, size_t srclen, cons
 		dst_encoding = "iso-8859-1"; // default character-set for most browsers
 
 	if (wget_strcasecmp_ascii(src_encoding, dst_encoding)) {
-		int ret = -1;
+		int ret = WGET_E_UNKNOWN;
 		iconv_t cd = iconv_open(dst_encoding, src_encoding);
 
 		if (cd != (iconv_t)-1) {
 			char *tmp = (char *) src; // iconv won't change where src points to, but changes tmp itself
 			size_t tmp_len = srclen;
 			size_t dst_len = tmp_len * 6, dst_len_tmp = dst_len;
-			char *dst = xmalloc(dst_len + 1), *dst_tmp = dst;
+			char *dst = wget_malloc(dst_len + 1), *dst_tmp = dst;
+
+			if (!dst) {
+				iconv_close(cd);
+				return WGET_E_MEMORY;
+			}
 
 			errno = 0;
 			if (iconv(cd, (ICONV_CONST char **)&tmp, &tmp_len, &dst_tmp, &dst_len_tmp) == 0
@@ -95,19 +94,27 @@ int wget_memiconv(const char *src_encoding, const void *src, size_t srclen, cons
 			{
 				debug_printf("transcoded %zu bytes from '%s' to '%s'\n", srclen, src_encoding, dst_encoding);
 				if (out) {
-					*out = xrealloc(dst, dst_len - dst_len_tmp + 1);
-					(*out)[dst_len - dst_len_tmp] = 0;
+					// here we reduce the allocated memory size, if it fails we use the original memory chunk
+					tmp = wget_realloc(dst, dst_len - dst_len_tmp + 1);
+					if (!tmp)
+						tmp = dst;
+					tmp[dst_len - dst_len_tmp] = 0;
+					*out = tmp;
 				} else
 					xfree(dst);
+
 				if (outlen)
 					*outlen = dst_len - dst_len_tmp;
-				ret = 0; // return OK
+
+				ret = WGET_E_SUCCESS;
 			} else {
 				// erno == 0 means some codepoints were encoded non-reversible, treat as error
 				error_printf(_("Failed to transcode '%s' string into '%s' (%d)\n"), src_encoding, dst_encoding, errno);
 				xfree(dst);
+
 				if (out)
 					*out = NULL;
+
 				if (outlen)
 					*outlen = 0;
 			}
@@ -122,10 +129,11 @@ int wget_memiconv(const char *src_encoding, const void *src, size_t srclen, cons
 
 	if (out)
 		*out = wget_strmemdup(src, srclen);
+
 	if (outlen)
 		*outlen = srclen;
 
-	return 0;
+	return WGET_E_SUCCESS;
 }
 
 // src must be a ASCII compatible C string
@@ -201,7 +209,7 @@ char *wget_utf8_to_str(const char *src, const char *encoding)
  * [2] https://lists.gnu.org/archive/html/bug-wget/2015-06/msg00002.html
  * [3] https://curl.haxx.se/mail/lib-2015-06/0143.html
  */
-static int G_GNUC_WGET_PURE _utf8_is_valid(const char *utf8)
+static int WGET_GCC_PURE _utf8_is_valid(const char *utf8)
 {
 	const unsigned char *s = (const unsigned char *) utf8;
 
@@ -228,48 +236,19 @@ static int G_GNUC_WGET_PURE _utf8_is_valid(const char *utf8)
 }
 #endif
 
+/* We convert hostnames and thus have to apply IDN2_USE_STD3_ASCII_RULES.
+ * If we don't do, the result could contain any ascii characters,
+ * e.g. 'evil.c\u2100.example.com' will be converted into
+ * 'evil.ca/c.example.com', which seems no good idea. */
 const char *wget_str_to_ascii(const char *src)
 {
 #ifdef WITH_LIBIDN2
 	if (wget_str_needs_encoding(src)) {
 		char *asc = NULL;
 		int rc;
-# if defined WITH_LIBUNISTRING && IDN2_VERSION_NUMBER < 0x00140000
-		uint8_t *lower, resbuf[256];
-		size_t len = sizeof(resbuf) - 1; // leave space for additional \0 byte
-
-		// we need a conversion to lowercase
-		lower = u8_tolower((uint8_t *)src, u8_strlen((uint8_t *)src), 0, UNINORM_NFKC, resbuf, &len);
-		if (!lower) {
-			error_printf(_("u8_tolower(%s) failed (%d)\n"), src, errno);
-			return src;
-		}
-
-		// u8_tolower() does not terminate the result string
-		if (lower == resbuf) {
-			lower[len]=0;
-		} else {
-			uint8_t *tmp = lower;
-			lower = (uint8_t *)wget_strmemdup((char *)lower, len);
-			xfree(tmp);
-		}
-
-		if ((rc = idn2_lookup_u8(lower, (uint8_t **)&asc, 0)) == IDN2_OK) {
-			debug_printf("idn2 '%s' -> '%s'\n", src, asc);
-			src = asc;
-		} else
-			error_printf(_("toASCII(%s) failed (%d): %s\n"), lower, rc, idn2_strerror(rc));
-
-		if (lower != resbuf)
-			xfree(lower);
-# else
-#  if IDN2_VERSION_NUMBER < 0x00140000
-		if ((rc = idn2_lookup_u8((uint8_t *)src, (uint8_t **)&asc, 0)) == IDN2_OK)
-#  else
-		if ((rc = idn2_lookup_u8((uint8_t *)src, (uint8_t **)&asc, IDN2_NONTRANSITIONAL)) != IDN2_OK)
-			rc = idn2_lookup_u8((uint8_t *)src, (uint8_t **)&asc, IDN2_TRANSITIONAL);
+		if ((rc = idn2_lookup_u8((uint8_t *)src, (uint8_t **)&asc, IDN2_NONTRANSITIONAL|IDN2_USE_STD3_ASCII_RULES)) != IDN2_OK)
+			rc = idn2_lookup_u8((uint8_t *)src, (uint8_t **)&asc, IDN2_TRANSITIONAL|IDN2_USE_STD3_ASCII_RULES);
 		if (rc == IDN2_OK)
-#  endif
 		{
 			debug_printf("idn2 '%s' -> '%s'\n", src, asc);
 #  ifdef _WIN32
@@ -280,7 +259,6 @@ const char *wget_str_to_ascii(const char *src)
 #  endif
 		} else
 			error_printf(_("toASCII(%s) failed (%d): %s\n"), src, rc, idn2_strerror(rc));
-# endif
 	}
 #elif defined WITH_LIBIDN
 	if (wget_str_needs_encoding(src)) {
