@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2017-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -30,12 +30,12 @@
 #include <wget.h>
 
 // _WGET_LOGGER is shared between log.c and logger.c, but must not be exposed to the public
-struct _wget_logger_st {
+struct wget_logger_st {
 	FILE *fp;
 	const char *fname;
 	void (*func)(const char *buf, size_t bufsize);
-	void (*vprintf)(const wget_logger_t *logger, const char *fmt, va_list args) G_GNUC_WGET_PRINTF_FORMAT(2,0);
-	void (*write)(const wget_logger_t *logger, const char *buf, size_t bufsize);
+	void (*vprintf)(const wget_logger *logger, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	void (*write)(const wget_logger *logger, const char *buf, size_t bufsize);
 };
 
 #endif /* LIBWGET_LOGGER_H */
