@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2015 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -30,6 +30,7 @@
 
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <netdb.h>
 
 struct wget_tcp_st {
 	void *
@@ -43,6 +44,8 @@ struct wget_tcp_st {
 	const char
 		*ssl_hostname, // if set, do SSL hostname checking
 		*ip;
+	wget_dns
+		*dns;
 	int
 		sockfd,
 		// timeouts in milliseconds
@@ -53,14 +56,11 @@ struct wget_tcp_st {
 		family,
 		preferred_family,
 		protocol; // WGET_PROTOCOL_HTTP1_1, WGET_PROTOCOL_HTTP2_0
-	wget_hpkp_stats_t
+	wget_hpkp_stats_result
 		hpkp; // hpkp stats
 
 	bool
 		ssl : 1,
-		caching : 1,
-		addrinfo_allocated : 1,
-		bind_addrinfo_allocated : 1,
 		tls_false_start : 1,
 		tcp_fastopen : 1, // do we use TCP_FASTOPEN or not
 		first_send : 1; // TCP_FASTOPEN's first packet is sent different
