@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -45,12 +45,12 @@
  */
 
 struct _sitemap_context {
-	wget_vector_t
+	wget_vector
 		*sitemap_urls,
 		*urls;
 };
 
-static void _sitemap_get_url(void *context, int flags, const char *dir, const char *attr G_GNUC_WGET_UNUSED, const char *val, size_t len, size_t pos G_GNUC_WGET_UNUSED)
+static void _sitemap_get_url(void *context, int flags, const char *dir, const char *attr WGET_GCC_UNUSED, const char *val, size_t len, size_t pos WGET_GCC_UNUSED)
 {
 	struct _sitemap_context *ctx = context;
 
@@ -69,19 +69,24 @@ static void _sitemap_get_url(void *context, int flags, const char *dir, const ch
 			for (;len && c_isspace(val[len - 1]); len--);  // skip trailing spaces
 
 			// info_printf("%02X %s %s '%.*s' %zd %zd\n", flags, dir, attr, (int) len, val, len, pos);
-			wget_string_t url = { .p = val, .len = len };
+			wget_string *url;
+
+			if (!(url = wget_malloc(sizeof(wget_string))))
+				return;
+
+			url->p = val;
+			url->len = len;
 
 			if (type == 1) {
 				if (!ctx->sitemap_urls)
 					ctx->sitemap_urls = wget_vector_create(32, NULL);
 
-				wget_vector_add(ctx->sitemap_urls, &url, sizeof(url));
+				wget_vector_add(ctx->sitemap_urls, url);
 			} else {
 				if (!ctx->urls)
 					ctx->urls = wget_vector_create(32, NULL);
 
-				wget_vector_add(ctx->urls, &url, sizeof(url));
-
+				wget_vector_add(ctx->urls, url);
 			}
 		}
 	}
@@ -96,7 +101,7 @@ static void _sitemap_get_url(void *context, int flags, const char *dir, const ch
  * as vector in \p urls and/or \p sitemap_urls.
  *
  */
-void wget_sitemap_get_urls_inline(const char *sitemap, wget_vector_t **urls, wget_vector_t **sitemap_urls)
+void wget_sitemap_get_urls_inline(const char *sitemap, wget_vector **urls, wget_vector **sitemap_urls)
 {
 	struct _sitemap_context context = { .urls = NULL, .sitemap_urls = NULL };
 
