@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2016-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2016-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -38,10 +38,10 @@
 #include <wget.h>
 #include "private.h"
 
-struct _wget_tls_session_db_st {
-	wget_hashmap_t *
+struct wget_tls_session_db_st {
+	wget_hashmap *
 		entries;
-	wget_thread_mutex_t
+	wget_thread_mutex
 		mutex;
 	int64_t
 		load_time;
@@ -49,7 +49,7 @@ struct _wget_tls_session_db_st {
 		changed : 1; // whether or not the db has been changed / needs saving
 };
 
-struct _wget_tls_session_st {
+struct wget_tls_session_st {
 	const char *
 		host;
 	int64_t
@@ -67,7 +67,8 @@ struct _wget_tls_session_st {
 #ifdef __clang__
 __attribute__((no_sanitize("integer")))
 #endif
-static unsigned int G_GNUC_WGET_PURE _hash_tls_session(const wget_tls_session_t *tls_session)
+WGET_GCC_PURE
+static unsigned int hash_tls_session(const wget_tls_session *tls_session)
 {
 	unsigned int hash = 0;
 	const unsigned char *p;
@@ -78,7 +79,8 @@ static unsigned int G_GNUC_WGET_PURE _hash_tls_session(const wget_tls_session_t
 	return hash;
 }
 
-static int G_GNUC_WGET_NONNULL_ALL G_GNUC_WGET_PURE _compare_tls_session(const wget_tls_session_t *s1, const wget_tls_session_t *s2)
+WGET_GCC_NONNULL_ALL WGET_GCC_PURE
+static int compare_tls_session(const wget_tls_session *s1, const wget_tls_session *s2)
 {
 	int n;
 
@@ -95,18 +97,20 @@ static int G_GNUC_WGET_NONNULL_ALL G_GNUC_WGET_PURE _compare_tls_session(const w
 	return 0;
 }
 
-wget_tls_session_t *wget_tls_session_init(wget_tls_session_t *tls_session)
+wget_tls_session *wget_tls_session_init(wget_tls_session *tls_session)
 {
-	if (!tls_session)
-		tls_session = xmalloc(sizeof(wget_tls_session_t));
+	if (!tls_session) {
+		if (!(tls_session = wget_calloc(1, sizeof(wget_tls_session))))
+			return NULL;
+	} else
+		memset(tls_session, 0, sizeof(*tls_session));
 
-	memset(tls_session, 0, sizeof(*tls_session));
 	tls_session->created = time(NULL);
 
 	return tls_session;
 }
 
-void wget_tls_session_deinit(wget_tls_session_t *tls_session)
+void wget_tls_session_deinit(wget_tls_session *tls_session)
 {
 	if (tls_session) {
 		xfree(tls_session->host);
@@ -114,7 +118,7 @@ void wget_tls_session_deinit(wget_tls_session_t *tls_session)
 	}
 }
 
-void wget_tls_session_free(wget_tls_session_t *tls_session)
+void wget_tls_session_free(wget_tls_session *tls_session)
 {
 	if (tls_session) {
 		wget_tls_session_deinit(tls_session);
@@ -122,33 +126,35 @@ void wget_tls_session_free(wget_tls_session_t *tls_session)
 	}
 }
 
-wget_tls_session_t *wget_tls_session_new(const char *host, time_t maxage, const void *data, size_t data_size)
+wget_tls_session *wget_tls_session_new(const char *host, time_t maxage, const void *data, size_t data_size)
 {
-	wget_tls_session_t *tls_session = wget_tls_session_init(NULL);
+	wget_tls_session *tls_session = wget_tls_session_init(NULL);
 
-	tls_session->host = wget_strdup(host);
-	tls_session->data = wget_memdup(data, data_size);
-	tls_session->data_size = data_size;
+	if (tls_session) {
+		tls_session->host = wget_strdup(host);
+		tls_session->data = wget_memdup(data, data_size);
+		tls_session->data_size = data_size;
 
-	if (maxage <= 0 || maxage >= INT64_MAX / 2 || tls_session->created < 0 || tls_session->created >= INT64_MAX / 2) {
-		tls_session->maxage = 0;
-		tls_session->expires = 0;
-	} else {
-		tls_session->maxage = maxage;
-		tls_session->expires = tls_session->created + maxage;
+		if (maxage <= 0 || maxage >= INT64_MAX / 2 || tls_session->created < 0 || tls_session->created >= INT64_MAX / 2) {
+			tls_session->maxage = 0;
+			tls_session->expires = 0;
+		} else {
+			tls_session->maxage = maxage;
+			tls_session->expires = tls_session->created + maxage;
+		}
 	}
 
 	return tls_session;
 }
 
-int wget_tls_session_get(const wget_tls_session_db_t *tls_session_db, const char *host, void **data, size_t *size)
+int wget_tls_session_get(const wget_tls_session_db *tls_session_db, const char *host, void **data, size_t *size)
 {
 	if (tls_session_db) {
-		wget_tls_session_t tls_session, *tls_sessionp;
+		wget_tls_session tls_session, *tls_sessionp;
 		int64_t now = time(NULL);
 
 		tls_session.host = host;
-		if ((tls_sessionp = wget_hashmap_get(tls_session_db->entries, &tls_session)) && tls_sessionp->expires >= now) {
+		if (wget_hashmap_get(tls_session_db->entries, &tls_session, &tls_sessionp) && tls_sessionp->expires >= now) {
 			if (data)
 				*data = wget_memdup(tls_sessionp->data, tls_sessionp->data_size);
 			if (size)
@@ -160,22 +166,31 @@ int wget_tls_session_get(const wget_tls_session_db_t *tls_session_db, const char
 	return 1;
 }
 
-wget_tls_session_db_t *wget_tls_session_db_init(wget_tls_session_db_t *tls_session_db)
+wget_tls_session_db *wget_tls_session_db_init(wget_tls_session_db *tls_session_db)
 {
-	if (!tls_session_db)
-		tls_session_db = xmalloc(sizeof(wget_tls_session_db_t));
+	wget_hashmap *entries = wget_hashmap_create(16, (wget_hashmap_hash_fn *) hash_tls_session, (wget_hashmap_compare_fn *) compare_tls_session);
 
-	memset(tls_session_db, 0, sizeof(*tls_session_db));
-	tls_session_db->entries = wget_hashmap_create(16, (wget_hashmap_hash_t)_hash_tls_session, (wget_hashmap_compare_t)_compare_tls_session);
-	wget_hashmap_set_key_destructor(tls_session_db->entries, (wget_hashmap_key_destructor_t)wget_tls_session_free);
-	wget_hashmap_set_value_destructor(tls_session_db->entries, (wget_hashmap_value_destructor_t)wget_tls_session_free);
+	if (!entries)
+		return NULL;
+
+	if (!tls_session_db) {
+		if (!(tls_session_db = wget_calloc(1, sizeof(wget_tls_session_db)))) {
+			wget_hashmap_free(&entries);
+			return NULL;
+		}
+	} else
+		memset(tls_session_db, 0, sizeof(*tls_session_db));
+
+	wget_hashmap_set_key_destructor(entries, (wget_hashmap_key_destructor *) wget_tls_session_free);
+	wget_hashmap_set_value_destructor(entries, (wget_hashmap_value_destructor *) wget_tls_session_free);
+	tls_session_db->entries = entries;
 
 	wget_thread_mutex_init(&tls_session_db->mutex);
 
 	return tls_session_db;
 }
 
-void wget_tls_session_db_deinit(wget_tls_session_db_t *tls_session_db)
+void wget_tls_session_db_deinit(wget_tls_session_db *tls_session_db)
 {
 	if (tls_session_db) {
 		wget_thread_mutex_lock(tls_session_db->mutex);
@@ -186,7 +201,7 @@ void wget_tls_session_db_deinit(wget_tls_session_db_t *tls_session_db)
 	}
 }
 
-void wget_tls_session_db_free(wget_tls_session_db_t **tls_session_db)
+void wget_tls_session_db_free(wget_tls_session_db **tls_session_db)
 {
 	if (tls_session_db) {
 		wget_tls_session_db_deinit(*tls_session_db);
@@ -194,8 +209,11 @@ void wget_tls_session_db_free(wget_tls_session_db_t **tls_session_db)
 	}
 }
 
-void wget_tls_session_db_add(wget_tls_session_db_t *tls_session_db, wget_tls_session_t *tls_session)
+void wget_tls_session_db_add(wget_tls_session_db *tls_session_db, wget_tls_session *tls_session)
 {
+	if (!tls_session_db || !tls_session)
+		return;
+
 	wget_thread_mutex_lock(tls_session_db->mutex);
 
 	if (tls_session->maxage == 0) {
@@ -206,31 +224,31 @@ void wget_tls_session_db_add(wget_tls_session_db_t *tls_session_db, wget_tls_ses
 		wget_tls_session_free(tls_session);
 		tls_session = NULL;
 	} else {
-		wget_tls_session_t *old = wget_hashmap_get(tls_session_db->entries, tls_session);
+		wget_tls_session *old;
 
-		if (old) {
+		if (wget_hashmap_get(tls_session_db->entries, tls_session, &old)) {
 			debug_printf("found TLS session data for %s\n", old->host);
 			if (wget_hashmap_remove(tls_session_db->entries, old))
 				debug_printf("removed TLS session data for %s\n", tls_session->host);
 		}
 
 		debug_printf("add TLS session data for %s (maxage=%lld, size=%zu)\n", tls_session->host, (long long)tls_session->maxage, tls_session->data_size);
-		wget_hashmap_put_noalloc(tls_session_db->entries, tls_session, tls_session);
+		wget_hashmap_put(tls_session_db->entries, tls_session, tls_session);
 		tls_session_db->changed = 1;
 	}
 
 	wget_thread_mutex_unlock(tls_session_db->mutex);
 }
 
-static int _tls_session_db_load(wget_tls_session_db_t *tls_session_db, FILE *fp)
+static int tls_session_db_load(wget_tls_session_db *tls_session_db, FILE *fp)
 {
-	wget_tls_session_t tls_session;
+	wget_tls_session tls_session;
 	struct stat st;
 	char *buf = NULL, *linep, *p;
 	size_t bufsize = 0;
 	ssize_t buflen;
 	int64_t now = time(NULL);
-	int ok;
+	bool ok;
 
 	// if the database file hasn't changed since the last read
 	// there's no need to reload
@@ -256,7 +274,7 @@ static int _tls_session_db_load(wget_tls_session_db_t *tls_session_db, FILE *fp)
 			buf[--buflen] = 0;
 
 		wget_tls_session_init(&tls_session);
-		ok = 0;
+		ok = false;
 
 		// parse host
 		if (*linep) {
@@ -295,15 +313,20 @@ static int _tls_session_db_load(wget_tls_session_db_t *tls_session_db, FILE *fp)
 				linep++;
 
 			size_t len = linep - p;
-			char *data = xmalloc(wget_base64_get_decoded_length(len));
-			tls_session.data_size = wget_base64_decode(data, p, len);
-			tls_session.data = data;
+			char *data = wget_malloc(wget_base64_get_decoded_length(len));
+			if (data) {
+				tls_session.data_size = wget_base64_decode(data, p, len);
+				tls_session.data = data;
 
-			ok = 1;
+				ok = true;
+			}
 		}
 
 		if (ok) {
+			bool no_change = wget_hashmap_size(tls_session_db->entries) == 0;
 			wget_tls_session_db_add(tls_session_db, wget_memdup(&tls_session, sizeof(tls_session)));
+			if (no_change)
+				tls_session_db->changed = 0;
 		} else {
 			wget_tls_session_deinit(&tls_session);
 			error_printf(_("Failed to parse HSTS line: '%s'\n"), buf);
@@ -323,40 +346,41 @@ static int _tls_session_db_load(wget_tls_session_db_t *tls_session_db, FILE *fp)
 // Load the TLS session cache from a flat file
 // Protected by flock()
 
-int wget_tls_session_db_load(wget_tls_session_db_t *tls_session_db, const char *fname)
+int wget_tls_session_db_load(wget_tls_session_db *tls_session_db, const char *fname)
 {
 	if (!tls_session_db || !fname || !*fname)
 		return 0;
 
-	if (wget_update_file(fname, (wget_update_load_t)_tls_session_db_load, NULL, tls_session_db)) {
+	if (wget_update_file(fname, (wget_update_load_fn *) tls_session_db_load, NULL, tls_session_db)) {
 		error_printf(_("Failed to read TLS session data\n"));
 		return -1;
 	} else {
-		debug_printf(_("Fetched TLS session data from '%s'\n"), fname);
+		debug_printf("Fetched TLS session data from '%s'\n", fname);
 		return 0;
 	}
 }
 
-static int G_GNUC_WGET_NONNULL_ALL _tls_session_save(FILE *fp, const wget_tls_session_t *tls_session)
+WGET_GCC_NONNULL_ALL
+static int tls_session_save(FILE *fp, const wget_tls_session *tls_session)
 {
-	char session_b64[((tls_session->data_size + 2) / 3) * 4 + 1];
+	char session_b64[wget_base64_get_encoded_length(tls_session->data_size)];
 
 	wget_base64_encode(session_b64, (const char *) tls_session->data, tls_session->data_size);
 
-	fprintf(fp, "%s %lld %lld %s\n", tls_session->host, (long long)tls_session->created, (long long)tls_session->maxage, session_b64);
+	wget_fprintf(fp, "%s %lld %lld %s\n", tls_session->host, (long long)tls_session->created, (long long)tls_session->maxage, session_b64);
 	return 0;
 }
 
-static int _tls_session_db_save(void *tls_session_db, FILE *fp)
+static int tls_session_db_save(void *tls_session_db, FILE *fp)
 {
-	wget_hashmap_t *entries = ((wget_tls_session_db_t *)tls_session_db)->entries;
+	wget_hashmap *entries = ((wget_tls_session_db *)tls_session_db)->entries;
 
 	if (wget_hashmap_size(entries) > 0) {
 		fputs("#TLSSession 1.0 file\n", fp);
 		fputs("#Generated by Wget2 " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("#<hostname>  <created> <max-age> <session data>\n\n", fp);
 
-		wget_hashmap_browse(entries, (wget_hashmap_browse_t)_tls_session_save, fp);
+		wget_hashmap_browse(entries, (wget_hashmap_browse_fn *) tls_session_save, fp);
 
 		if (ferror(fp))
 			return -1;
@@ -368,29 +392,29 @@ static int _tls_session_db_save(void *tls_session_db, FILE *fp)
 // Save the TLS session cache to a flat file
 // Protected by flock()
 
-int wget_tls_session_db_save(wget_tls_session_db_t *tls_session_db, const char *fname)
+int wget_tls_session_db_save(wget_tls_session_db *tls_session_db, const char *fname)
 {
 	int size;
 
 	if (!tls_session_db || !fname || !*fname)
 		return -1;
 
-	if (wget_update_file(fname, (wget_update_load_t)_tls_session_db_load, _tls_session_db_save, tls_session_db)) {
+	if (wget_update_file(fname, (wget_update_load_fn *) tls_session_db_load, tls_session_db_save, tls_session_db)) {
 		error_printf(_("Failed to write TLS session file '%s'\n"), fname);
 		return -1;
 	}
 
 	if ((size = wget_hashmap_size(tls_session_db->entries)))
-		debug_printf(_("Saved %d TLS session entr%s into '%s'\n"), size, size != 1 ? "ies" : "y", fname);
+		debug_printf("Saved %d TLS session entr%s into '%s'\n", size, size != 1 ? "ies" : "y", fname);
 	else
-		debug_printf(_("No TLS session entries to save. Table is empty.\n"));
+		debug_printf("No TLS session entries to save. Table is empty.\n");
 
 	tls_session_db->changed = 0;
 
 	return 0;
 }
 
-int wget_tls_session_db_changed(wget_tls_session_db_t *tls_session_db)
+int wget_tls_session_db_changed(wget_tls_session_db *tls_session_db)
 {
 	return tls_session_db ? tls_session_db->changed : 0;
 }
