@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -34,19 +34,20 @@
 #include <wget.h>
 #include "private.h"
 
-struct _wget_vector_st {
-	wget_vector_compare_t
-		cmp; // comparison function
-	wget_vector_destructor_t
-		destructor; // element destructor function
+struct wget_vector_st {
+	wget_vector_compare_fn
+		*cmp; //!< comparison function for sorting and searching
+	wget_vector_destructor
+		*destructor; //!< element destructor function
 	void
-		**entry; // pointer to array of pointers to elements
+		**entry; //!< pointer to array of pointers to elements
 	int
-		max,     // allocated elements
-		cur,     // number of elements in use
-		off;     // number of elements to add if resize occurs
+		max,     //!< allocated elements
+		cur;     //!< number of elements in use
 	bool
-		sorted : 1; // 1=list is sorted, 0=list is not sorted
+		sorted : 1; //!< 1=list is sorted, 0=list is not sorted
+	float
+		resize_factor; //!< factor to calculate new vector size
 };
 
 /**
@@ -65,63 +66,69 @@ struct _wget_vector_st {
  *
  * Create a new vector instance, to be free'd after use with wget_vector_free().
  */
-wget_vector_t *wget_vector_create(int max, wget_vector_compare_t cmp)
+wget_vector *wget_vector_create(int max, wget_vector_compare_fn *cmp)
 {
-	wget_vector_t *v = xcalloc(1, sizeof(wget_vector_t));
+	wget_vector *v = wget_calloc(1, sizeof(wget_vector));
+
+	if (!v)
+		return NULL;
+
+	if (!(v->entry = wget_malloc(max * sizeof(void *)))) {
+		xfree(v);
+		return NULL;
+	}
 
-	v->entry = xmalloc(max * sizeof(void *));
 	v->max = max;
-	v->off = -2;
+	v->resize_factor = 2;
 	v->cmp = cmp;
+	v->destructor = free;
 
 	return v;
 }
 
 /**
  * \param[in] v Vector
- * \param[in] off Vector growth mode:
- *   positive values: increase vector by \p off entries on each resize
- *   negative values: increase vector by multiplying \p -off, e.g. -2 doubles the size on each resize
+ * \param[in] factor Vector growth factor
  *
- * Set the growth policy for internal memory.
+ * Set the factor for resizing the vector when it is full.
+ *
+ * The new size is 'factor * oldsize'. If the new size is less or equal the old size,
+ * the involved insertion function will return an error and the internal state of
+ * the vector will not change.
+ *
+ * Default is 2.
  */
-void wget_vector_set_growth_policy(wget_vector_t *v, int off)
+void wget_vector_set_resize_factor(wget_vector *v, float factor)
 {
 	if (v)
-		v->off = off;
+		v->resize_factor = factor;
 }
 
-static int G_GNUC_WGET_NONNULL((2)) _vec_insert_private(wget_vector_t *v, const void *elem, size_t size, int pos, int replace, int alloc)
+static int WGET_GCC_NONNULL((2)) insert_element(wget_vector *v, const void *elem, int pos, int replace)
 {
-	void *elemp;
-
-	if (pos < 0 || !v || pos > v->cur) return -1;
-
-	if (alloc) {
-		elemp = xmalloc(size);
-		memcpy(elemp, elem, size);
-	} else {
-		elemp = (void *)elem;
-	}
+	if (pos < 0 || !v || pos > v->cur)
+		return WGET_E_INVALID;
 
 	if (!replace) {
 		if (v->max == v->cur) {
-			if (v->off > 0) {
-				v->entry = xrealloc(v->entry, (v->max += v->off) * sizeof(void *));
-			} else if (v->off<-1) {
-				v->entry = xrealloc(v->entry, (v->max *= -v->off) * sizeof(void *));
-			} else {
-				if (alloc)
-					free(elemp);
-				return -1;
-			}
+			int newsize = (int) (v->max * v->resize_factor);
+
+			if (newsize <= v->max)
+				return WGET_E_INVALID;
+
+			void **tmp = wget_realloc(v->entry, newsize * sizeof(void *));
+			if (!tmp)
+				return WGET_E_MEMORY;
+
+			v->entry = tmp;
+			v->max = newsize;
 		}
 
 		memmove(&v->entry[pos + 1], &v->entry[pos], (v->cur - pos) * sizeof(void *));
 		v->cur++;
 	}
 
-	v->entry[pos] = elemp;
+	v->entry[pos] = (void *) elem;
 
 	if (v->cmp) {
 		if (v->cur == 1) v->sorted = 1;
@@ -145,26 +152,8 @@ static int G_GNUC_WGET_NONNULL((2)) _vec_insert_private(wget_vector_t *v, const
 /**
  * \param[in] v Vector where \p elem is inserted into
  * \param[in] elem Element to insert into \p v
- * \param[in] size Size of \p elem
  * \param[in] pos Position to insert \p elem at
- * \return Index of inserted element or -1 on error
- *
- * Insert \p elem of given \p size at index \p pos.
- *
- * \p elem is cloned / copied (shallow).
- *
- * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
- */
-int wget_vector_insert(wget_vector_t *v, const void *elem, size_t size, int pos)
-{
-	return _vec_insert_private(v, elem, size, pos, 0, 1);
-}
-
-/**
- * \param[in] v Vector where \p elem is inserted into
- * \param[in] elem Element to insert into \p v
- * \param[in] pos Position to insert \p elem at
- * \return Index of inserted element or -1 on error
+ * \return Index of inserted element (>= 0) or WGET_E_*  on error (< 0)
  *
  * Insert \p elem of at index \p pos.
  *
@@ -172,17 +161,31 @@ int wget_vector_insert(wget_vector_t *v, const void *elem, size_t size, int pos)
  *
  * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
  */
-int wget_vector_insert_noalloc(wget_vector_t *v, const void *elem, int pos)
+int wget_vector_insert(wget_vector *v, const void *elem, int pos)
 {
-	return _vec_insert_private(v, elem, 0, pos, 0, 0);
+	return insert_element(v, elem, pos, 0);
 }
 
-static int G_GNUC_WGET_NONNULL((2)) _vec_insert_sorted_private(wget_vector_t *v, const void *elem, size_t size, int alloc)
+/**
+ * \param[in] v Vector where \p elem is inserted into
+ * \param[in] elem Element to insert into \p v
+ * \return Index of inserted element (>= 0) or WGET_E_*  on error (< 0)
+ *
+ * Insert \p elem of at a position that keeps the sort order of the elements.
+ * If the vector has no comparison function, \p elem will be inserted as the last element.
+ * If the elements in the vector are not sorted, they will be sorted after returning from this function.
+ *
+ * \p elem is *not* cloned, the vector takes 'ownership' of the element.
+ *
+ * An error is returned if \p v is %NULL.
+ */
+int wget_vector_insert_sorted(wget_vector *v, const void *elem)
 {
-	if (!v) return -1;
+	if (!v)
+		return WGET_E_INVALID;
 
 	if (!v->cmp)
-		return _vec_insert_private(v, elem, size, v->cur, 0, alloc);
+		return insert_element(v, elem, v->cur, 0);
 
 	if (!v->sorted)
 		wget_vector_sort(v);
@@ -194,55 +197,18 @@ static int G_GNUC_WGET_NONNULL((2)) _vec_insert_sorted_private(wget_vector_t *v,
 		m = (l + r) / 2;
 		if ((res = v->cmp(elem, v->entry[m])) > 0) l = m + 1;
 		else if (res < 0) r = m - 1;
-		else return _vec_insert_private(v, elem, size, m, 0, alloc);
+		else return insert_element(v, elem, m, 0);
 	}
 	if (res > 0) m++;
 
-	return _vec_insert_private(v, elem, size, m, 0, alloc);
-}
-
-/**
- * \param[in] v Vector where \p elem is inserted into
- * \param[in] elem Element to insert into \p v
- * \param[in] size Size of \p elem
- * \return Index of inserted element or -1 on error
- *
- * Insert \p elem of given \p size at a position that keeps the sort order of the elements.
- * If the vector has no comparison function, \p elem will be inserted as the last element.
- * If the elements in the vector are not sorted, they will be sorted after returning from this function.
- *
- * \p elem is cloned / copied (shallow).
- *
- * An error is returned if \p v is %NULL.
- */
-int wget_vector_insert_sorted(wget_vector_t *v, const void *elem, size_t size)
-{
-	return _vec_insert_sorted_private(v, elem, size, 1);
-}
-
-/**
- * \param[in] v Vector where \p elem is inserted into
- * \param[in] elem Element to insert into \p v
- * \return Index of inserted element or -1 on error
- *
- * Insert \p elem of at a position that keeps the sort order of the elements.
- * If the vector has no comparison function, \p elem will be inserted as the last element.
- * If the elements in the vector are not sorted, they will be sorted after returning from this function.
- *
- * \p elem is *not* cloned, the vector takes 'ownership' of the element.
- *
- * An error is returned if \p v is %NULL.
- */
-int wget_vector_insert_sorted_noalloc(wget_vector_t *v, const void *elem)
-{
-	return _vec_insert_sorted_private(v, elem, 0, 0);
+	return insert_element(v, elem, m, 0);
 }
 
 /**
  * \param[in] v Vector where \p elem is appended to
  * \param[in] elem Element to append to a \p v
  * \param[in] size Size of \p elem
- * \return Index of appended element or -1 on error
+ * \return Index of inserted element (>= 0) or WGET_E_*  on error (< 0)
  *
  * Append \p elem of given \p size to vector \p v.
  *
@@ -250,15 +216,27 @@ int wget_vector_insert_sorted_noalloc(wget_vector_t *v, const void *elem)
  *
  * An error is returned if \p v is %NULL.
  */
-int wget_vector_add(wget_vector_t *v, const void *elem, size_t size)
+int wget_vector_add_memdup(wget_vector *v, const void *elem, size_t size)
 {
-	return v ? _vec_insert_private(v, elem, size, v->cur, 0, 1) : -1;
+	void *elemp;
+	int rc;
+
+	if (!v)
+		return WGET_E_INVALID;
+
+	if (!(elemp = wget_memdup(elem, size)))
+		return WGET_E_MEMORY;
+
+	if ((rc = insert_element(v, elemp, v->cur, 0)) < 0)
+		xfree(elemp);
+
+	return rc;
 }
 
 /**
  * \param[in] v Vector where \p elem is appended to
  * \param[in] elem Element to append to a \p v
- * \return Index of appended element or -1 on error
+ * \return Index of inserted element (>= 0) or WGET_E_*  on error (< 0)
  *
  * Append \p elem to vector \p v.
  *
@@ -266,104 +244,65 @@ int wget_vector_add(wget_vector_t *v, const void *elem, size_t size)
  *
  * An error is returned if \p v is %NULL.
  */
-int wget_vector_add_noalloc(wget_vector_t *v, const void *elem)
+int wget_vector_add(wget_vector *v, const void *elem)
 {
-	return v ? _vec_insert_private(v, elem, 0, v->cur, 0, 0) : -1;
-}
-
-/**
- * \param[in] v Vector where \p s is appended to
- * \param[in] s String to append to \p v
- * \return Index of appended element or -1 on error
- *
- * Append string \p s as an element to vector \p v.
- *
- * \p s is cloned / copied.
- *
- * An error is returned if \p v or \p s is %NULL.
- */
-int wget_vector_add_str(wget_vector_t *v, const char *s)
-{
-	return v && s ? _vec_insert_private(v, s, strlen(s) + 1, v->cur, 0, 1) : -1;
+	return v ? insert_element(v, elem, v->cur, 0) : WGET_E_INVALID;
 }
 
 /**
  * \param[in] v Vector where \p s is appended to
  * \param[in] fmt Printf-like format string
  * \param[in] args Arguments for the \p fmt
- * \return Index of appended element or -1 on error
+ * \return Index of appended element (>= 0) or WGET_E_* on error (< 0)
  *
  * Construct string in a printf-like manner and append it as an element to vector \p v.
  *
  * An error is returned if \p v or \p fmt is %NULL.
  */
-int wget_vector_add_vprintf(wget_vector_t *v, const char *fmt, va_list args)
+int wget_vector_add_vprintf(wget_vector *v, const char *fmt, va_list args)
 {
-	return v && fmt ? _vec_insert_private(v, wget_vaprintf(fmt, args), 0, v->cur, 0, 0) : -1;
+	if (!v || !fmt)
+		return WGET_E_INVALID;
+
+	char *p = wget_vaprintf(fmt, args);
+	if (!p)
+		return WGET_E_MEMORY;
+
+	return insert_element(v, p, v->cur, 0);
 }
 
 /**
  * \param[in] v Vector where \p s is appended to
  * \param[in] fmt Printf-like format string
  * \param[in] ... Arguments for the \p fmt
- * \return Index of appended element or -1 on error
+ * \return Index of appended element (>= 0) or WGET_E_* on error (< 0)
  *
  * Construct string in a printf-like manner and append it as an element to vector \p v.
  *
  * An error is returned if \p v or \p fmt is %NULL.
  */
-int wget_vector_add_printf(wget_vector_t *v, const char *fmt, ...)
+int wget_vector_add_printf(wget_vector *v, const char *fmt, ...)
 {
 	if (!v || !fmt)
-		return -1;
+		return WGET_E_INVALID;
 
 	va_list args;
 
 	va_start(args, fmt);
-	int pos = _vec_insert_private(v, wget_vaprintf(fmt, args), 0, v->cur, 0, 0);
+	char *p = wget_vaprintf(fmt, args);
 	va_end(args);
 
-	return pos;
-}
+	if (!p)
+		return WGET_E_MEMORY;
 
-static int _wget_vector_replace(wget_vector_t *v, const void *elem, size_t size, int pos, int alloc)
-{
-	if (!v || pos < 0 || pos >= v->cur)
-		return -1;
-
-	if (v->destructor)
-		v->destructor(v->entry[pos]);
-
-	xfree(v->entry[pos]);
-
-	return _vec_insert_private(v, elem, size, pos, 1, alloc); // replace existing entry
-}
-
-/**
- * \param[in] v Vector where \p elem is inserted
- * \param[in] elem Element to insert into \p v
- * \param[in] size Size of \p elem
- * \param[in] pos Position to insert \p elem at
- * \return Index of inserted element (same as \p pos) or -1 on error
- *
- * Replace the element at position \p pos with \p elem of given \p size.
- * If the vector has an element destructor function, this is called.
- * The old element is free'd.
- *
- * \p elem is cloned / copied (shallow).
- *
- * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
- */
-int wget_vector_replace(wget_vector_t *v, const void *elem, size_t size, int pos)
-{
-	return _wget_vector_replace(v, elem, size, pos, 1);
+	return insert_element(v, p, v->cur, 0);
 }
 
 /**
  * \param[in] v Vector where \p elem is inserted
  * \param[in] elem Element to insert into \p v
  * \param[in] pos Position to insert \p elem at
- * \return Index of inserted element (same as \p pos) or -1 on error
+ * \return Index of inserted element (same as \p pos) (>= 0) or WGET_E_* on error (< 0)
  *
  * Replace the element at position \p pos with \p elem.
  * If the vector has an element destructor function, this is called.
@@ -373,21 +312,25 @@ int wget_vector_replace(wget_vector_t *v, const void *elem, size_t size, int pos
  *
  * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
  */
-int wget_vector_replace_noalloc(wget_vector_t *v, const void *elem, int pos)
+int wget_vector_replace(wget_vector *v, const void *elem, int pos)
 {
-	return _wget_vector_replace(v, elem, 0, pos, 0);
+	if (!v || pos < 0 || pos >= v->cur)
+		return WGET_E_INVALID;
+
+	if (v->destructor)
+		v->destructor(v->entry[pos]);
+
+	return insert_element(v, elem, pos, 1); // replace existing entry
 }
 
-static int _vec_remove_private(wget_vector_t *v, int pos, int free_entry)
+static int remove_element(wget_vector *v, int pos, int free_entry)
 {
 	if (pos < 0 || !v || pos >= v->cur)
-		return -1;
+		return WGET_E_INVALID;
 
 	if (free_entry) {
 		if (v->destructor)
 			v->destructor(v->entry[pos]);
-
-		xfree(v->entry[pos]);
 	}
 
 	memmove(&v->entry[pos], &v->entry[pos + 1], (v->cur - pos - 1) * sizeof(void *));
@@ -399,7 +342,7 @@ static int _vec_remove_private(wget_vector_t *v, int pos, int free_entry)
 /**
  * \param[in] v Vector to remove an element from
  * \param[in] pos Position of element to remove
- * \return Index of removed element (same as \p pos) or -1 on error
+ * \return Index of appended element (>= 0) or WGET_E_* on error (< 0)
  *
  * Remove the element at position \p pos.
  * If the vector has an element destructor function, this is called.
@@ -407,31 +350,31 @@ static int _vec_remove_private(wget_vector_t *v, int pos, int free_entry)
  *
  * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
  */
-int wget_vector_remove(wget_vector_t *v, int pos)
+int wget_vector_remove(wget_vector *v, int pos)
 {
-	return _vec_remove_private(v, pos, 1);
+	return remove_element(v, pos, 1);
 }
 
 /**
  * \param[in] v Vector to remove an element from
  * \param[in] pos Position of element to remove
- * \return Index of removed element (same as \p pos) or -1 on error
+ * \return Index of removed element (same as \p pos) (>= 0) or WGET_E_* on error (< 0)
  *
  * Remove the element at position \p pos.
  * No element destructor function is called, the element is not free'd.
  *
  * An error is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
  */
-int wget_vector_remove_nofree(wget_vector_t *v, int pos)
+int wget_vector_remove_nofree(wget_vector *v, int pos)
 {
-	return _vec_remove_private(v, pos, 0);
+	return remove_element(v, pos, 0);
 }
 
 /**
  * \param[in] v Vector to act on
  * \param[in] old_pos Position to move element from
  * \param[in] new_pos Position to move element to
- * \return Index of new position (same as \p new_pos) or -1 on error
+ * \return Index of new position (same as \p new_pos) (>= 0) or WGET_E_* on error (< 0)
  *
  * Move the element at position \p old_pos to \p new_pos.
  *
@@ -440,11 +383,11 @@ int wget_vector_remove_nofree(wget_vector_t *v, int pos)
  * An error is returned if \p v is %NULL or either \p old_pos or
  * \p new_pos is out of range (< 0 or > # of entries).
  */
-int wget_vector_move(wget_vector_t *v, int old_pos, int new_pos)
+int wget_vector_move(wget_vector *v, int old_pos, int new_pos)
 {
-	if (!v) return -1;
-	if (old_pos < 0 || old_pos >= v->cur) return -1;
-	if (new_pos < 0 || new_pos >= v->cur) return -1;
+	if (!v) return WGET_E_INVALID;
+	if (old_pos < 0 || old_pos >= v->cur) return WGET_E_INVALID;
+	if (new_pos < 0 || new_pos >= v->cur) return WGET_E_INVALID;
 	if (old_pos == new_pos) return new_pos;
 
 	if (v->sorted && v->cmp && v->cmp(v->entry[old_pos], v->entry[new_pos]))
@@ -467,18 +410,18 @@ int wget_vector_move(wget_vector_t *v, int old_pos, int new_pos)
  * \param[in] v Vector to act on
  * \param[in] pos1 Position of element one
  * \param[in] pos2 Position of element two
- * \return Index of second position (same as \p pos2) or -1 on error
+ * \return Index of second position (same as \p pos2) (>= 0) or WGET_E_*  on error (< 0)
  *
  * Swap the two elements at position \p pos1 and \p pos2.
  *
  * An error is returned if \p v is %NULL or either \p pos1 or
  * \p pos2 is out of range (< 0 or > # of entries).
  */
-int wget_vector_swap(wget_vector_t *v, int pos1, int pos2)
+int wget_vector_swap(wget_vector *v, int pos1, int pos2)
 {
-	if (!v) return -1;
-	if (pos1 < 0 || pos1 >= v->cur) return -1;
-	if (pos2 < 0 || pos2 >= v->cur) return -1;
+	if (!v) return WGET_E_INVALID;
+	if (pos1 < 0 || pos1 >= v->cur) return WGET_E_INVALID;
+	if (pos2 < 0 || pos2 >= v->cur) return WGET_E_INVALID;
 	if (pos1 == pos2) return pos2;
 
 	void *tmp = v->entry[pos1];
@@ -499,7 +442,7 @@ int wget_vector_swap(wget_vector_t *v, int pos1, int pos2)
  * For each element the destructor function is called and the element free'd thereafter.
  * Then the vector itself is free'd and set to %NULL.
  */
-void wget_vector_free(wget_vector_t **v)
+void wget_vector_free(wget_vector **v)
 {
 	if (v && *v) {
 		wget_vector_clear(*v);
@@ -516,17 +459,14 @@ void wget_vector_free(wget_vector_t **v)
  * For each element the destructor function is called and the element free'd thereafter.
  * The vector is then empty and can be reused.
  */
-void wget_vector_clear(wget_vector_t *v)
+void wget_vector_clear(wget_vector *v)
 {
 	if (v) {
 		if (v->destructor) {
 			for (int it = 0; it < v->cur; it++) {
 				v->destructor(v->entry[it]);
-				xfree(v->entry[it]);
+				v->entry[it] = NULL;
 			}
-		} else {
-			for (int it = 0; it < v->cur; it++)
-				xfree(v->entry[it]);
 		}
 
 		v->cur = 0;
@@ -541,7 +481,7 @@ void wget_vector_clear(wget_vector_t *v)
  *
  * The vector is then empty and can be reused.
  */
-void wget_vector_clear_nofree(wget_vector_t *v)
+void wget_vector_clear_nofree(wget_vector *v)
 {
 	if (v) {
 		for (int it = 0; it < v->cur; it++)
@@ -557,7 +497,7 @@ void wget_vector_clear_nofree(wget_vector_t *v)
  * Retrieve the number of elements of the vector \p v.
  * If \p v is %NULL, 0 is returned.
  */
-int wget_vector_size(const wget_vector_t *v)
+int wget_vector_size(const wget_vector *v)
 {
 	return v ? v->cur : 0;
 }
@@ -571,7 +511,7 @@ int wget_vector_size(const wget_vector_t *v)
  *
  * %NULL is returned if \p v is %NULL or \p pos is out of range (< 0 or > # of entries).
  */
-void *wget_vector_get(const wget_vector_t *v, int pos)
+void *wget_vector_get(const wget_vector *v, int pos)
 {
 	if (pos < 0 || !v || pos >= v->cur)
 		return NULL;
@@ -592,7 +532,7 @@ void *wget_vector_get(const wget_vector_t *v, int pos)
  *
  * The return value of the last call to \p browse is returned or 0 if \p v is %NULL.
  */
-int wget_vector_browse(const wget_vector_t *v, wget_vector_browse_t browse, void *ctx)
+int wget_vector_browse(const wget_vector *v, wget_vector_browse_fn *browse, void *ctx)
 {
 	if (v) {
 		for (int ret, it = 0; it < v->cur; it++)
@@ -609,7 +549,7 @@ int wget_vector_browse(const wget_vector_t *v, wget_vector_browse_t browse, void
  *
  * Set the compare function used by wget_vector_sort().
  */
-void wget_vector_setcmpfunc(wget_vector_t *v, wget_vector_compare_t cmp)
+void wget_vector_setcmpfunc(wget_vector *v, wget_vector_compare_fn *cmp)
 {
 	if (v) {
 		v->cmp = cmp;
@@ -628,27 +568,28 @@ void wget_vector_setcmpfunc(wget_vector_t *v, wget_vector_compare_t cmp)
  * Set the destructor function that is called for each element to be removed.
  * It should not free the element (pointer) itself.
  */
-void wget_vector_set_destructor(wget_vector_t *v, wget_vector_destructor_t destructor)
+void wget_vector_set_destructor(wget_vector *v, wget_vector_destructor *destructor)
 {
 	if (v)
 		v->destructor = destructor;
 }
 
-static int G_GNUC_WGET_NONNULL_ALL _compare(const void *p1, const void *p2, void *v)
+WGET_GCC_NONNULL_ALL
+static int compare_element(const void *p1, const void *p2, void *v)
 {
-	return ((wget_vector_t *)v)->cmp(*((void **)p1), *((void **)p2));
+	return ((wget_vector *)v)->cmp(*((void **)p1), *((void **)p2));
 }
 
 /**
  * \param[in] v Vector
  *
- * Sort the elements in vector \p v usign the compare function.
+ * Sort the elements in vector \p v using the compare function.
  * Do nothing if \p v is %NULL or the compare function is not set.
  */
-void wget_vector_sort(wget_vector_t *v)
+void wget_vector_sort(wget_vector *v)
 {
 	if (v && v->cmp) {
-		qsort_r(v->entry, v->cur, sizeof(void *), _compare, v);
+		qsort_r(v->entry, v->cur, sizeof(void *), compare_element, v);
 		v->sorted = 1;
 	}
 }
@@ -656,43 +597,43 @@ void wget_vector_sort(wget_vector_t *v)
 /**
  * \param[in] v Vector
  * \param[in] elem Element to search for
- * \return Index of the found element or -1 if not found
+ * \return Index of the found element, WGET_E_UNKNOWN if not found or WGET_E_INVALID
+ *         if v was %NULL or there was no comparison function set
  *
  * Searches for the given element using the compare function of the vector.
- *
- * Returns -1 if \p v is %NULL or if the compare function is not set.
  */
-int wget_vector_find(const wget_vector_t *v, const void *elem)
+int wget_vector_find(const wget_vector *v, const void *elem)
 {
-	if (v && v->cmp) {
-		if (v->cur == 1) {
-			if (v->cmp(elem, v->entry[0]) == 0) return 0;
-		} else if (v->sorted) {
-			// binary search for element (exact match)
-			for (int l = 0, r = v->cur - 1; l <= r;) {
-				int res, m = (l + r) / 2;
-				if ((res = v->cmp(elem, v->entry[m])) > 0) l = m + 1;
-				else if (res < 0) r = m - 1;
-				else return m;
-			}
-		} else {
-			// linear search for element
-			for (int it = 0; it < v->cur; it++)
-				if (v->cmp(elem, v->entry[it]) == 0) return it;
+	if (!v || !v->cmp)
+		return WGET_E_INVALID;
+
+	if (v->cur == 1) {
+		if (v->cmp(elem, v->entry[0]) == 0) return 0;
+	} else if (v->sorted) {
+		// binary search for element (exact match)
+		for (int l = 0, r = v->cur - 1; l <= r;) {
+			int res, m = (l + r) / 2;
+			if ((res = v->cmp(elem, v->entry[m])) > 0) l = m + 1;
+			else if (res < 0) r = m - 1;
+			else return m;
 		}
+	} else {
+		// linear search for element
+		for (int it = 0; it < v->cur; it++)
+			if (v->cmp(elem, v->entry[it]) == 0) return it;
 	}
 
-	return -1; // not found
+	return WGET_E_UNKNOWN; // not found
 }
 
 /**
  * \param[in] v Vector
  * \param[in] elem Element to check for
- * \return 1 if element exists, else 0
+ * \return True if element exists, else false
  *
  * Checks whether the element \p elem exists or not.
  */
-int wget_vector_contains(const wget_vector_t *v, const void *elem)
+bool wget_vector_contains(const wget_vector *v, const void *elem)
 {
 	return wget_vector_find(v, elem) >= 0;
 }
@@ -702,31 +643,32 @@ int wget_vector_contains(const wget_vector_t *v, const void *elem)
  * \param[in] start Index to start search from
  * \param[in] direction Direction of search
  * \param[in] find Function to be called for each element
- * \return Index of the found element or -1 if not found
+ * \return Index of the found element, WGET_E_UNKNOWN if not found or WGET_E_INVALID
+ *         if v was %NULL or there was no comparison function set
  *
  * Call \p find for each element starting at \p start.
  * If \p find returns 0 the current index is returned.
- *
- * Returns -1 if \p v is %NULL or if the \p find didn't return 0.
  */
-int wget_vector_findext(const wget_vector_t *v, int start, int direction, wget_vector_find_t find)
+int wget_vector_findext(const wget_vector *v, int start, int direction, wget_vector_find_fn *find)
 {
+	if (!v)
+		return WGET_E_INVALID;
 
-	if (v) {
-		if (direction) { // down
-			if (start < v->cur) {
-				for (int it = start; it >= 0; it--)
-					if (find(v->entry[it]) == 0) return it;
-			}
-		} else { // up
-			if (start >= 0) {
-				for (int it = start; it < v->cur; it++)
-					if (find(v->entry[it]) == 0) return it;
-			}
+	if (direction) { // down
+		if (start < v->cur) {
+			for (int it = start; it >= 0; it--)
+				if (find(v->entry[it]) == 0)
+					return it;
+		}
+	} else { // up
+		if (start >= 0) {
+			for (int it = start; it < v->cur; it++)
+				if (find(v->entry[it]) == 0)
+					return it;
 		}
 	}
 
-	return -1;
+	return WGET_E_UNKNOWN;
 }
 
 /**@}*/
