@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -59,8 +59,8 @@ typedef struct {
 		token_len; // used bytes of token buffer (not counting terminating 0 byte)
 	void
 		*user_ctx; // user context (not needed if we were using nested functions)
-	wget_xml_callback_t
-		callback;
+	wget_xml_callback
+		*callback;
 } _xml_context;
 
 /* \cond _hide_internal_symbols */
@@ -502,7 +502,7 @@ static int parseXML(const char *dir, _xml_context *context)
  */
 int wget_xml_parse_buffer(
 	const char *buf,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
@@ -530,7 +530,7 @@ int wget_xml_parse_buffer(
  */
 void wget_html_parse_buffer(
 	const char *buf,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
@@ -549,7 +549,7 @@ void wget_html_parse_buffer(
  */
 void wget_xml_parse_file(
 	const char *fname,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
@@ -563,7 +563,7 @@ void wget_xml_parse_file(
 				size_t nread = st.st_size;
 				char *buf = mmap(NULL, nread + 1, PROT_READ|PROT_WRITE, MAP_PRIVATE, fd, 0);
 #else
-				char *buf=xmalloc(st.st_size + 1);
+				char *buf=wget_malloc(st.st_size + 1);
 				size_t nread=read(fd, buf, st.st_size);
 #endif
 
@@ -586,16 +586,18 @@ void wget_xml_parse_file(
 		// maybe should use yy_scan_bytes instead of buffering into memory.
 		char tmp[4096];
 		ssize_t nbytes;
-		wget_buffer_t *buf = wget_buffer_alloc(4096);
+		wget_buffer buf;
+
+		wget_buffer_init(&buf, NULL, 4096);
 
 		while ((nbytes = read(STDIN_FILENO, tmp, sizeof(tmp))) > 0) {
-			wget_buffer_memcat(buf, tmp, nbytes);
+			wget_buffer_memcat(&buf, tmp, nbytes);
 		}
 
-		if (buf->length)
-			wget_xml_parse_buffer(buf->data, callback, user_ctx, hints);
+		if (buf.length)
+			wget_xml_parse_buffer(buf.data, callback, user_ctx, hints);
 
-		wget_buffer_free(&buf);
+		wget_buffer_deinit(&buf);
 	}
 }
 
@@ -611,7 +613,7 @@ void wget_xml_parse_file(
  */
 void wget_html_parse_file(
 	const char *fname,
-	wget_xml_callback_t callback,
+	wget_xml_callback *callback,
 	void *user_ctx,
 	int hints)
 {
