@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2014 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -47,14 +47,14 @@
 // for which the thread will sleep before waking up and redrawing the progress
 enum { _BAR_THREAD_SLEEP_DURATION = 125 };
 
-static wget_bar_t
+static wget_bar
 	*bar;
-static wget_thread_t
+static wget_thread
 	progress_thread;
 static bool
 	_terminate_thread;
 
-static void *_bar_update_thread(void *p G_GNUC_WGET_UNUSED)
+static void *_bar_update_thread(void *p WGET_GCC_UNUSED)
 {
 	while (!_terminate_thread) {
 		wget_bar_update(bar);
@@ -92,7 +92,7 @@ bool bar_init(void)
 	}
 
 nobar:
-	wget_error_printf(_("Cannot create progress bar thread. Disabling progess bar.\n"));
+	wget_error_printf(_("Cannot create progress bar thread. Disabling progress bar.\n"));
 	config.progress = 0;
 	return false;
 }
@@ -123,9 +123,9 @@ void bar_printf(int slot, const char *fmt, ...)
 	va_end(args);
 }
 
-void bar_slot_begin(int slot, const char *filename, ssize_t filesize)
+void bar_slot_begin(int slot, const char *filename, int new_file, ssize_t filesize)
 {
-	wget_bar_slot_begin(bar, slot, filename, filesize);
+	wget_bar_slot_begin(bar, slot, filename, new_file, filesize);
 }
 
 void bar_set_downloaded(int slot, size_t nbytes)
