@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2017-2019 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -57,7 +57,7 @@ void dl_error_set(dl_error_t *e, const char *msg);
 
 // Set an error message with printf format.
 void dl_error_set_printf
-	(dl_error_t *e, const char *format, ...) G_GNUC_WGET_PRINTF_FORMAT(2, 3);
+	(dl_error_t *e, const char *format, ...) WGET_GCC_PRINTF_FORMAT(2, 3);
 
 // Returns 1 if dynamic loader will work on the current platform, 0 otherwise
 int dl_supported(void);
@@ -90,10 +90,10 @@ char *dl_get_name_from_path(const char *path, int strict);
  * directories. If found it returns the filename, else returns NULL.
  * Free the returned string with wget_free().
  */
-char *dl_search(const char *name, const wget_vector_t *dirs);
+char *dl_search(const char *name, const wget_vector *dirs);
 
 /* Creates a list of loadable object files in a given list of directories.
  */
-void dl_list(const wget_vector_t *dirs, wget_vector_t *names_out);
+void dl_list(const wget_vector *dirs, wget_vector *names_out);
 
 #endif /* SRC_WGET_DL_H */
