@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -37,19 +37,20 @@ typedef struct JOB JOB;
 // everything host/domain specific should go here
 typedef struct {
 	const char
-		*scheme,
 		*host;
 	JOB
 		*robot_job; // special job for downloading robots.txt (before anything else)
-	wget_robots_t
+	wget_robots
 		*robots;
-	wget_list_t
+	wget_list
 		*queue; // host specific job queue
 	long long
 		retry_ts; // timestamp of earliest retry in milliseconds
 	int
 		qsize, // number of jobs in queue
 		failures; // number of consequent connection failures
+	wget_iri_scheme
+		scheme;
 	uint16_t
 		port;
 	bool
@@ -59,22 +60,22 @@ typedef struct {
 void host_init(void);
 void host_exit(void);
 
-HOST *host_add(wget_iri_t *iri) G_GNUC_WGET_NONNULL((1));
-HOST *host_get(wget_iri_t *iri) G_GNUC_WGET_NONNULL((1));
+HOST *host_add(wget_iri *iri) WGET_GCC_NONNULL((1));
+HOST *host_get(wget_iri *iri) WGET_GCC_NONNULL((1));
 
 JOB *host_get_job(HOST *host, long long *pause);
-void host_add_job(HOST *host, const JOB *job) G_GNUC_WGET_NONNULL((1,2));
-void host_add_robotstxt_job(HOST *host, wget_iri_t *iri, bool http_fallback) G_GNUC_WGET_NONNULL((1,2));
+void host_add_job(HOST *host, const JOB *job) WGET_GCC_NONNULL((1,2));
+void host_add_robotstxt_job(HOST *host, wget_iri *iri, bool http_fallback) WGET_GCC_NONNULL((1,2));
 void host_release_jobs(HOST *host);
-void host_remove_job(HOST *host, JOB *job) G_GNUC_WGET_NONNULL((1,2));
-void host_queue_free(HOST *host) G_GNUC_WGET_NONNULL((1));
+void host_remove_job(HOST *host, JOB *job) WGET_GCC_NONNULL((1,2));
+void host_queue_free(HOST *host) WGET_GCC_NONNULL((1));
 void hosts_free(void);
-void host_increase_failure(HOST *host) G_GNUC_WGET_NONNULL((1));
-void host_final_failure(HOST *host) G_GNUC_WGET_NONNULL((1));
-void host_reset_failure(HOST *host) G_GNUC_WGET_NONNULL((1));
+void host_increase_failure(HOST *host) WGET_GCC_NONNULL((1));
+void host_final_failure(HOST *host) WGET_GCC_NONNULL((1));
+void host_reset_failure(HOST *host) WGET_GCC_NONNULL((1));
 
-int queue_size(void) G_GNUC_WGET_PURE;
-int queue_empty(void) G_GNUC_WGET_PURE;
+int queue_size(void) WGET_GCC_PURE;
+int queue_empty(void) WGET_GCC_PURE;
 void queue_print(HOST *host);
 
 #endif /* SRC_WGET_HOST_H */
