@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -53,11 +53,11 @@
 #define debug_write wget_debug_write
 
 // I try to never leave freed pointers hanging around
-#define xfree(a) do { if (a) { free((void *)(a)); a=NULL; } } while (0)
+#define xfree(a) do { if (a) { wget_free((void *)(a)); a=NULL; } } while (0)
 
 // number of elements within an array
 #define countof(a) (sizeof(a)/sizeof(*(a)))
 
-const char * G_GNUC_WGET_NONNULL_ALL get_local_filename(const wget_iri_t *iri);
+const char * WGET_GCC_NONNULL_ALL get_local_filename(const wget_iri *iri);
 
 #endif /* SRC_WGET_MAIN_H */
