@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2018 Free Software Foundation, Inc.
+ * Copyright(c) 2018-2019 Free Software Foundation, Inc.
  *
  * This file is part of Wget.
  *
@@ -25,6 +25,8 @@
 
 #include <wget.h>
 
+void mkdir_path(const char *fname, bool is_file);
 char *shell_expand(const char *fname);
+char *wget_restrict_file_name(const char *fname, char *esc, int mode);
 
 #endif /* SRC_WGET_UTILS_H */
