@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -79,9 +79,8 @@ int main(void)
 		0);
 
 	// test-idn-meta
-	snprintf(options, sizeof(options),
-		"--iri -rH -e http_proxy=localhost:%d http://start-here.com/start.html",
-		wget_test_get_http_server_port());
+	wget_snprintf(options, sizeof(options),
+		"--iri -rH -e http_proxy=localhost:{{port}} -e https_proxy=localhost:{{sslport}} http://start-here.com/start.html");
 
 	wget_test(
 //		WGET_TEST_KEEP_TMPFILES, 1,
@@ -110,10 +109,8 @@ int main(void)
 		0);
 
 	// test-idn-meta with HTML5 meta charset
-	snprintf(options, sizeof(options),
-		"--iri -rH -e http_proxy=localhost:%d http://start-here.com/start2.html",
-		wget_test_get_http_server_port());
-
+	wget_snprintf(options, sizeof(options),
+		"--iri -rH -e http_proxy=localhost:{{port}} -e https_proxy=localhost:{{sslport}} http://start-here.com/start2.html");
 	wget_test(
 //		WGET_TEST_KEEP_TMPFILES, 1,
 		WGET_TEST_OPTIONS, options,
