@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -73,9 +73,8 @@ int main(void)
 		0);
 
 	// test-idn-robots
-	snprintf(options, sizeof(options),
-		"--iri -e robots=on -rH -e http_proxy=localhost:%d --local-encoding=EUC-JP http://" euc_jp_hostname "/",
-		wget_test_get_http_server_port());
+	wget_snprintf(options, sizeof(options),
+		"--iri -e robots=on -rH -e http_proxy=localhost:{{port}} -e https_proxy=localhost:{{sslport}} --local-encoding=EUC-JP http://" euc_jp_hostname "/");
 
 	wget_test(
 //		WGET_TEST_KEEP_TMPFILES, 1,
@@ -92,9 +91,8 @@ int main(void)
 // UTF-8 command line characters are mangled on MinGW on C locale
 #ifndef _WIN32
 	// test-idn-robots-utf8
-	snprintf(options, sizeof(options),
-		"--iri -e robots=on -rH -e http_proxy=localhost:%d --local-encoding=UTF-8 http://" utf8_hostname "/",
-		wget_test_get_http_server_port());
+	wget_snprintf(options, sizeof(options),
+		"--iri -e robots=on -rH -e http_proxy=localhost:{{port}} -e https_proxy=localhost:{{sslport}} --local-encoding=UTF-8 http://" utf8_hostname "/");
 
 	urls[0].body = "<a href=\"http://" utf8_hostname "/foo.txt\">The link</a>";
 	urls[0].headers[0] = "Content-Type: text/html; charset=UTF-8";
