@@ -1,6 +1,6 @@
 /*
  * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2015-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -95,9 +95,8 @@ int main(void)
 		0);
 
 	char options[128];
-	snprintf(options, sizeof(options),
-		"--force-css --base http://localhost:%d -i main.css",
-		wget_test_get_http_server_port());
+	wget_snprintf(options, sizeof(options),
+		"--force-css --base http://localhost:{{port}} -i main.css");
 
 	// test--parse-css from file
 	wget_test(
