@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright(c) 2017-2019 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -27,19 +27,10 @@
 #include <string.h> // strlen()
 #include <unistd.h> // access()
 #include "libtest.h"
+#include "plugin_tests.h"
 
 // #define LARGEFILE (11 << 20)
 
-#define OBJECT_DIR "../.libs"
-
-#if defined _WIN32
-#define LOCAL_NAME(x) OBJECT_DIR "/lib" x ".dll"
-#elif defined __CYGWIN__
-#define LOCAL_NAME(x) OBJECT_DIR "/cyg" x ".dll"
-#else
-#define LOCAL_NAME(x) OBJECT_DIR "/lib" x ".so"
-#endif
-
 static const char *mainpage = "\
 <html>\n\
 <head>\n\
